from django.db import models


class Author(models.Model):
    name = models.CharField(max_length=200)


class Book(models.Model):
    name = models.CharField(max_length=200)
    author = models.ForeignKey(Author)



#!/usr/bin/env python
""" Settings to run library tests
"""
import logging
import sys


logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s',
    datefmt='%d.%m %H:%M:%S')


from django.conf import settings as django_settings
django_settings.configure(
    ADMINS=('test', 'test@test.com'),
    ROOT_URLCONF='tests.core.urls',
    DEBUG=True,

    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': ':memory:',
            'USER': '',
            'PASSWORD': '',
            'TEST_CHARSET': 'utf8',
        }
    },
    CACHE_BACKEND='locmem://',

    INSTALLED_APPS=(
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'adrest',
        'tests.core', 'tests.main', 'tests.rpc', 'tests.simple'
    ),

    TEMPLATE_DEBUG=True,
    TEMPLATE_CONTEXT_PROCESSORS = (
        'django.core.context_processors.static',
        'django.core.context_processors.request',
        'django.contrib.auth.context_processors.auth',
    ),

    ADREST_ACCESS_LOG=True,
    ADREST_ALLOW_OPTIONS=True,
    ADREST_MAIL_ERRORS=(500, 400),
    ADREST_AUTO_CREATE_ACCESSKEY=True,
)


from django.test.simple import DjangoTestSuiteRunner
tests_runner = DjangoTestSuiteRunner(failfast=False)
sys.exit(tests_runner.run_tests(['core']))



from ..settings import ALLOW_OPTIONS
from ..utils import status
from ..utils.auth import AnonimousAuthenticator, AbstractAuthenticator
from ..utils.exceptions import HttpError
from ..utils.tools import as_tuple


__all__ = 'AuthMixin',


class AuthMeta(type):
    """ Prepare and checks resource.authenticators.
    """
    def __new__(mcs, name, bases, params):
        cls = super(AuthMeta, mcs).__new__(mcs, name, bases, params)
        cls.authenticators = as_tuple(cls.authenticators)
        for a in cls.authenticators:
            assert issubclass(a, AbstractAuthenticator), \
                "{0}.authenticators should be subclasses \
                    of `adrest.utils.auth.AbstractAuthenticator`"
        return cls


class AuthMixin(object):
    " Adds pluggable authentication behaviour "

    __metaclass__ = AuthMeta

    authenticators = AnonimousAuthenticator

    def __init__(self, *args, **kwargs):
        super(AuthMixin, self).__init__(*args, **kwargs)
        self.auth = None

    def authenticate(self, request):
        """ Attempt to authenticate the request, returning an authentication
            context or None.

            An authentication context may be any object, although in many cases
            it will simply be a :class:`User` instance.
        """
        authenticators = self.authenticators

        if request.method == 'OPTIONS' and ALLOW_OPTIONS:
            self.auth = AnonimousAuthenticator(self)
            return True

        error_message = "Authorization required."
        for authenticator in authenticators:
            auth = authenticator(self)
            try:
                assert auth.authenticate(request), error_message
                self.auth = auth
                auth.configure(request)
                return True
            except AssertionError, e:
                error_message = str(e)

        raise HttpError(error_message, status=status.HTTP_401_UNAUTHORIZED)

    def check_rights(self, resources, request=None):
        " Check rights of client for queried resources "
        if not self.auth:
            return True

        try:
            assert self.auth.test_rights(resources, request=request)
        except AssertionError, e:
            raise HttpError(
                "Access forbiden. {0}".format(e),
                status=status.HTTP_403_FORBIDDEN
            )



from urllib import urlencode

from django.core.paginator import InvalidPage, Paginator as DjangoPaginator

from .exceptions import HttpError
from .status import HTTP_400_BAD_REQUEST


class Paginator(object):
    """ Paginate collections.
    """
    def __init__(self, request, qs, max_res):
        self.query_dict = dict(request.GET.items())
        self._page = None
        self.path = request.path

        try:
            self.paginator = DjangoPaginator(
                qs, self.query_dict.get('max') or max_res)
            assert self.paginator.per_page
        except (ValueError, AssertionError):
            self.paginator = None

    def to_simple(self, serializer=None):
        return dict(
            count=self.paginator.count,
            page=self.page,
            num_pages=self.paginator.num_pages,
            next=self.next_page,
            prev=self.previous_page,
            resources=self.resources,
        )

    @property
    def page(self):
        if not self._page:
            try:
                self._page = self.paginator.page(
                    self.query_dict.get('page', 1))
            except InvalidPage:
                raise HttpError("Invalid page", status=HTTP_400_BAD_REQUEST)
        return self._page

    @property
    def count(self):
        return self.paginator.count

    @property
    def resources(self):
        return self.page.object_list

    @property
    def next_page(self):
        if self.page.has_next():
            self.query_dict['page'] = self.page.next_page_number()
            return "%s?%s" % (self.path, urlencode(self.query_dict))
        return ""

    @property
    def previous_page(self):
        if self.page.has_previous():
            previous = self.page.previous_page_number()
            if previous == 1:
                if 'page' in self.query_dict:
                    del self.query_dict['page']
            else:
                self.query_dict['page'] = previous
            return "%s?%s" % (self.path, urlencode(self.query_dict))
        return ""



class MetaOptions(object):
    " Resource meta options "

    def __init__(self):
        self.name = ''
        self.url_name = ''
        self.url_regex = ''
        self.parents = []
        self.model_fields = set()

        self.parsers_dict = dict()
        self.default_parser = None

    def __str__(self):
        return "%(url_name)s(%(name)s) - %(url_regex)s %(parents)s".format(
            self.__dict__
        )

    __repr__ = __str__


class UpdatedList(list):

    @property
    def len(self):
        return len(self)



from django.http import QueryDict, HttpResponse
from django.utils import simplejson, importlib

from ..utils.emitter import JSONPEmitter, JSONEmitter
from ..utils.parser import JSONParser, FormParser
from ..utils.tools import as_tuple
from ..utils.response import SerializedHttpResponse
from ..views import ResourceView, ResourceMetaClass


__all__ = 'get_request', 'RPCResource', 'AutoJSONRPC'


def get_request(func):
    """ Mark function as needed in request.
    """
    func.request = True
    return func


class RPCMeta(ResourceMetaClass):
    """ Setup RPC methods by Scheme.
    """
    def __new__(mcs, name, bases, params):
        cls = super(RPCMeta, mcs).__new__(mcs, name, bases, params)
        cls.configure_rpc()
        return cls


class RPCResource(ResourceView):
    """
        JSON RPC support.
        -----------------

        Implementation of remote procedure call encoded in JSON.
        Allows for notifications (info sent to the server that does not require
        a response) and for multiple calls to be sent to the server which may
        be answered out of order.

    """

    allowed_methods = 'get', 'post'
    emitters = JSONEmitter, JSONPEmitter
    methods = dict()
    parsers = JSONParser, FormParser
    scheme = None
    scheme_name = ''
    url_regex = r'^rpc$'

    __metaclass__ = RPCMeta

    def __init__(self, scheme=None, **kwargs):
        if scheme:
            self.configure_rpc(scheme)
        super(RPCResource, self).__init__(**kwargs)

    @classmethod
    def configure_rpc(cls, scheme=None):
        """ Get method from scheme.
        """
        scheme = scheme or cls.scheme
        if not cls.scheme:
            return False

        if isinstance(scheme, basestring):
            scheme = importlib.import_module(scheme)

        cls.scheme_name = scheme.__name__

        methods = getattr(scheme, '__all__', None) \
            or [m for m in dir(scheme) if not m.startswith('_')]

        for mname in methods:
            method = getattr(scheme, mname)
            if hasattr(method, '__call__'):
                cls.methods["{0}.{1}".format(
                    cls.scheme_name, method.__name__)] = method

    def handle_request(self, request, **resources):

        if request.method == 'OPTIONS':
            return super(RPCResource, self).handle_request(
                request, **resources)

        payload = request.data

        try:

            if request.method == 'GET':
                payload = request.GET.get('payload')
                try:
                    payload = simplejson.loads(payload)
                except TypeError:
                    raise AssertionError("Invalid RPC Call.")

            assert 'method' in payload, "Invalid RPC Call."
            return self.rpc_call(request, **payload)

        except Exception, e:
            return SerializedHttpResponse(
                dict(error=dict(message=str(e))),
                error=True
            )

    def rpc_call(self, request, method=None, params=None, **kwargs):
        args = []
        kwargs = dict()
        if isinstance(params, dict):
            kwargs.update(params)
        else:
            args = list(as_tuple(params))

        method_key = "{0}.{1}".format(self.scheme_name, method)
        assert method_key in self.methods, "Unknown method: {0}".format(method)
        method = self.methods[method_key]

        if hasattr(method, 'request'):
            args.insert(0, request)

        return method(*args, **kwargs)


class AutoJSONRPC(RPCResource):
    """
        Automatic JSONRPC Api from REST
        -------------------------------

        Automatic Implementation of remote procedure call based on your REST.

    """
    separator = '.'
    url_name = 'autojsonrpc'

    @staticmethod
    def configure_rpc(scheme=None):
        pass

    def rpc_call(self, request, method=None, **payload):
        """ Call REST API with RPC force.
        """
        assert method and self.separator in method, \
            "Wrong method name: {0}".format(method)

        resource_name, method = method.split(self.separator, 1)
        assert resource_name in self.api.resources, "Unknown method"

        data = QueryDict('', mutable=True)
        data.update(payload.get('data', dict()))
        data['callback'] = payload.get('callback') or request.GET.get(
            'callback') or request.GET.get('jsonp') or 'callback'
        for h, v in payload.get('headers', dict()).iteritems():
            request.META["HTTP_%s" % h.upper().replace('-', '_')] = v

        request.POST = request.PUT = request.GET = data
        delattr(request, '_request')
        request.method = method.upper()
        request.META['CONTENT_TYPE'] = 'application/x-www-form-urlencoded'
        params = payload.pop('params', dict())
        response = self.api.call(resource_name, request, **params)

        # Make response for prevent serialization
        new_response = HttpResponse(response.content,
                                    status=response.status_code)
        new_response._headers = response._headers # nolint
        assert new_response.status_code == 200, new_response.content
        return new_response


# pymode:lint_ignore=E1103,W0703



from django.conf.urls.defaults import include, patterns
from ..rpc.api import API as rpc
from ..simple.api import API as simple
from ..main.api import API as main
from ..main.resources import DummyResource


urlpatterns = main.urls + patterns(
    '',
    DummyResource.as_url(),
    (r'^simple/', include(simple.urls)),
    (r'^rpc/', include(rpc.urls)),
)



from pyga.requests import Tracker, Visitor, Session, Page

from ..views import ResourceView


__all__ = 'GaResource',


class GaResource(ResourceView):
    """
        Google Analytics support.
        -------------------------

        Track GA from server.

        Example: ::

            api.register(GaResource, account_id="UA-123456", domain="test")

    """

    url_regex = r'^ga(?P<path>.*)$'
    domain = None
    account_id = None

    def get(self, request, path=None, **resources):
        tracker = Tracker(
            self.account_id, self.domain or request.META.get('SERVER_NAME'))
        visitor = Visitor()
        visitor.extract_from_server_meta(request.META)
        session = Session()
        page = Page(path)
        tracker.track_pageview(page, session, visitor)



from adrest.api import Api
from adrest.resources.rpc import RPCResource

from . import rpc
from .resources import * # nolint


api = Api(version='0.1')

api.register(AuthorResource)
api.register(BookResource)
api.register(RPCResource, scheme=rpc)



from StringIO import StringIO
from urlparse import urlparse

from django.core.urlresolvers import reverse
from django.db.models import Model
from django.test import TestCase, Client
from django.utils import simplejson
from django.utils.functional import curry


MULTIPART_CONTENT = 'multipart/form-data; boundary=BoUnDaRyStRiNg'


class AdrestClient(Client):

    def patch(self, path, data=None, content_type=MULTIPART_CONTENT, follow=False, **extra):  # nolint
        " Send a resource to the server using PATCH. "

        data = data or dict()
        patch_data = self._encode_data(data, content_type)
        parsed = urlparse(path)
        r = {
            'CONTENT_LENGTH': len(patch_data),
            'CONTENT_TYPE': content_type,
            'PATH_INFO': self._get_path(parsed),
            'QUERY_STRING': parsed[4],
            'REQUEST_METHOD': 'PATCH',
            'wsgi.input': FakePayload(patch_data),
        }
        r.update(extra)
        response = self.request(**r)
        if follow:
            response = self._handle_redirects(response, **extra)
        return response


class AdrestTestCase(TestCase):
    api = None

    def setUp(self):
        assert self.api, "AdrestTestCase must have the api attribute."
        self.client = AdrestClient()

    def reverse(self, resource, **kwargs):
        """ Reverse resource by ResourceClass or name.

            :param resource: Resource Class or String name.
            :param **kwargs: Uri params
        """
        if isinstance(resource, basestring):
            url_name = resource
            assert self.api.resources.get(
                url_name), "Invalid resource name: %s" % url_name

        else:
            url_name = resource.meta.url_name

        kwargs = dict((k, getattr(v, "pk", v)) for k, v in kwargs.iteritems())
        name_ver = '' if not str(self.api) else '%s-' % str(self.api)
        return reverse(
            '%s-%s%s' % (self.api.prefix, name_ver, url_name), kwargs=kwargs)

    def get_params(self, resource, headers=None, data=None, key=None, **kwargs):  # nolint
        headers = headers or dict()
        data = data or dict()
        if isinstance(key, Model):
            key = key.key
        headers['HTTP_AUTHORIZATION'] = key or headers.get(
            'HTTP_AUTHORIZATION')
        resource = self.reverse(resource, **kwargs)
        return resource, headers, data

    def get_resource(self, resource, method='get', data=None, headers=None, json=False, **kwargs):  # nolint
        """ Simply run resource method.

            :param resource: Resource Class or String name.
            :param data: Request data
            :param json: Make JSON request
            :param headers: Request headers
            :param key: HTTP_AUTHORIZATION token
        """
        method = getattr(self.client, method)
        resource, headers, data = self.get_params(
            resource, headers, data, **kwargs)

        # Support JSON request
        if json:
            headers['content_type'] = 'application/json'
            data = simplejson.dumps(data)

        return method(resource, data=data, **headers)

    def rpc(self, resource, rpc=None, headers=None, callback=None, **kwargs):
        """ Emulate RPC call.

            :param resource: Resource Class or String name.
            :param rpc: RPC params.
            :param headers: Send headers
            :param callback: JSONP callback
        """
        resource, headers, data = self.get_params(
            resource, headers, data=rpc, **kwargs)

        if callback:
            headers['HTTP_ACCEPT'] = 'text/javascript'
            method = self.client.get
            data = dict(
                callback=callback,
                payload=simplejson.dumps(data))

        else:
            headers['HTTP_ACCEPT'] = 'application/json'
            method = self.client.post
            data = simplejson.dumps(data)

        return method(
            resource, data=data, content_type='application/json', **headers)

    put_resource = curry(get_resource, method='put')
    post_resource = curry(get_resource, method='post')
    patch_resource = curry(get_resource, method='patch')
    delete_resource = curry(get_resource, method='delete')


class FakePayload(object):
    """
    A wrapper around StringIO that restricts what can be read since data from
    the network can't be seeked and cannot be read outside of its content
    length. This makes sure that views can't do anything under the test client
    that wouldn't work in Real Life.
    """
    def __init__(self, content):
        self.__content = StringIO(content)
        self.__len = len(content)

    def read(self, num_bytes=None):
        if num_bytes is None:
            num_bytes = self.__len or 0
        assert self.__len >= num_bytes, "Cannot read more than the available bytes from the HTTP incoming data."  # nolint
        content = self.__content.read(num_bytes)
        self.__len -= num_bytes
        return content



from ...simple.tests import * # nolint
from ...rpc.tests import * # nolint
from ...main.tests import * # nolint

from .serializer import * # nolint
from .handler import * # nolint
from .resource import * # nolint
from .api import * # nolint



from django.test import TestCase
from milkman.dairy import milkman


class CoreSerializerTest(TestCase):

    def setUp(self):
        for _ in range(1, 10):
            milkman.deliver('main.book')

    def test_base_types(self):
        """ Testing serialization of base types.
        """
        from adrest.utils.serializer import BaseSerializer
        try:
            from collections import OrderedDict
        except ImportError:
            from ordereddict import OrderedDict # nolint

        from datetime import datetime
        from decimal import Decimal

        serializer = BaseSerializer()
        data = dict(
            string_='test',
            unicode_=unicode('test'),
            datetime_=datetime(2007, 01, 01),
            odict_=OrderedDict(value=1),
            dict_=dict(
                list_=[1, 2.35, Decimal(3), False]
            )
        )

        value = serializer.serialize(data)
        self.assertEqual(value, dict(
            string_=u'test',
            unicode_=u'test',
            datetime_='2007-01-01T00:00:00',
            odict_=dict(value=1),
            dict_=dict(
                list_=[1, 2.35, 3.0, False]
            )
        ))

    def test_django_model(self):
        from adrest.utils.serializer import BaseSerializer
        user = milkman.deliver('auth.User', username="testusername")
        data = [
            milkman.deliver('simple.task', user=user),
            milkman.deliver('simple.task', user=user),
            28, 'string']

        serializer = BaseSerializer(
            exclude='fake',
            include='username',
            related=dict(
                user=dict(fields='email')
            ),
        )
        self.assertEqual(serializer.model_options['exclude'], set(['fake']))

        out = serializer.to_simple(data, **serializer.model_options)
        self.assertEqual(out[0]['fields']['username'], data[0].user.username)

        # Test m2o serialization
        serializer = BaseSerializer(
            include="task_set",
            related=dict(
                task_set=dict(
                    fields=[])
            ),
        )
        out = serializer.to_simple(user, **serializer.model_options)

        self.assertEquals(len(out['fields']['task_set']), 2)
        for task in out['fields']['task_set']:
            self.assertEquals(task['fields']['user'], user.pk)
            self.assertTrue('title' in task['fields'].keys())

        out = serializer.to_simple(user)
        self.assertTrue('model' in out)

        out = serializer.to_simple(user, include=['task_set'])
        self.assertTrue(out['fields']['task_set'])

    def test_xml(self):
        from adrest.utils.serializer import XMLSerializer
        from ...main.models import Book
        for _ in range(1, 10):
            milkman.deliver(Book)
        worker = XMLSerializer()
        test = worker.serialize(Book.objects.all())
        self.assertTrue("author" in test)

    def test_json(self):
        from ...main.models import Author
        from adrest.utils.serializer import JSONSerializer
        authors = Author.objects.all()
        worker = JSONSerializer(options=dict(
            separators=(',', ':')
        ))
        test = worker.serialize(authors)
        self.assertTrue("main.author" in test)
        self.assertTrue('"fields":{"active":true,"name"' in test)



import abc
import base64

from django.contrib.auth import authenticate


class AbstractAuthenticator(object):
    " Abstract base authenticator "

    __meta__ = abc.ABCMeta

    def __init__(self, resource):
        self.resource = resource
        self.identifier = ''

    @abc.abstractmethod
    def authenticate(self, request):
        raise NotImplementedError

    @abc.abstractmethod
    def configure(self, request):
        raise NotImplementedError

    @staticmethod
    def test_rights(resources, request=None):
        return True

    @staticmethod
    def get_fields():
        return []


class AnonimousAuthenticator(AbstractAuthenticator):
    " Anonymous access. Set identifier by IP address. "

    def authenticate(self, request):
        return True

    def configure(self, request):
        pass


class UserLoggedInAuthenticator(AbstractAuthenticator):
    " Check auth by session. "

    def __init__(self, *args, **kwargs):
        self.user = None
        super(UserLoggedInAuthenticator, self).__init__(*args, **kwargs)

    def authenticate(self, request):
        user = getattr(request, 'user', None)
        return user and user.is_active

    def configure(self, request):
        self.user = request.user
        self.resource.identifier = self.user.username


class BasicAuthenticator(UserLoggedInAuthenticator):
    " HTTP Basic authentication. "

    def authenticate(self, request=None):
        if 'HTTP_AUTHORIZATION' in request.META:
            auth = request.META['HTTP_AUTHORIZATION'].split()
            if len(auth) == 2 and auth[0].lower() == "basic":
                uname, passwd = base64.b64decode(auth[1]).split(':')
                user = authenticate(username=uname, password=passwd)
                if user and user.is_active:
                    request.user = user
                    return True

        return False


class UserAuthenticator(UserLoggedInAuthenticator):
    " Authorization by login and password "

    username_fieldname = 'username'
    password_fieldname = 'password'

    def authenticate(self, request=None):
        try:
            username = request.REQUEST.get(self.username_fieldname)
            password = request.REQUEST.get(self.password_fieldname)
            request.user = authenticate(username=username, password=password)
            return request.user and request.user.is_active

        except KeyError:
            return False

    @classmethod
    def get_fields(cls):
        return [(cls.username_fieldname, dict(required=True)), (cls.password_fieldname, dict(required=True))]


try:
    from ..models import AccessKey

    class AccessKeyAuthenticator(UserLoggedInAuthenticator):
        " Authorization by API token. "

        def authenticate(self, request=None):
            """ Authenticate user using AccessKey from HTTP Header or GET params.
            """
            try:
                token = request.META.get('HTTP_AUTHORIZATION') or request.REQUEST['key']
                accesskey = AccessKey.objects.select_related('user').get(key=token)
                request.user = accesskey.user
                return request.user and request.user.is_active

            except(KeyError, AccessKey.DoesNotExist):
                return False

except ImportError:
    pass



version_info = (2, 1, 13)

__version__ = version = '.'.join(map(str, version_info))
__project__ = PROJECT = __name__
__author__ = AUTHOR = "Kirill Klenov <horneds@gmail.com>"
__license__ = LICENSE = "GNU LGPL"

try:
    from django.conf import settings as django_settings # nolint
    if django_settings.configured \
       and not 'adrest' in django_settings.INSTALLED_APPS:
        import logging
        logging.warn('You should add "adrest" to INSTALLED_APPS.')
except ImportError:
    pass



import logging

from django.conf.urls.defaults import patterns
from django.dispatch import Signal
from django.http import HttpRequest

from .resources.map import MapResource
from .resources.rpc import AutoJSONRPC
from .views import ResourceView
from .utils import exceptions, status, tools, emitter


__all__ = 'Api',


logger = logging.getLogger('adrest')


class Api(object):

    """ Implements a registry to tie together the various resources that make
        up an API.

        Especially useful for navigation, providing multiple versions of your
        API.

        :param version: Version info as string or iterable.
        :param api_map: Enable API map (true by default)
        :param api_prefix: API Url prefix ('api' by default)
        :param api_rpc: Enable automatic Json RPC (default false)

        Additional params will be putted in self resources.

    """
    def __init__(self, version=None, api_map=True, api_prefix='api',
                 api_rpc=False, **params):
        self.version = self.str_version = version
        self.prefix = api_prefix
        self.params = params
        self.resources = dict()
        self.request_started = Signal()
        self.request_finished = Signal()

        if api_map:
            self.resources[MapResource.meta.url_name] = MapResource

        # Enable Auto JSON RPC resource
        if api_rpc:
            self.resources[AutoJSONRPC.meta.url_name] = AutoJSONRPC
            self.params['emitters'] = tools.as_tuple(
                params.get('emitters', [])) + (
                    emitter.JSONPEmitter, emitter.JSONEmitter
                )

        if not isinstance(self.str_version, basestring):
            try:
                self.str_version = '.'.join(map(str, version or list()))
            except TypeError:
                self.str_version = str(version)

    def __str__(self):
        return self.str_version

    def register(self, resource, **params):
        """ Registers resource for the API.

            :param resource: Resource class for registration

            Additional params will be putted in the resource.
        """

        # Must be instance of ResourceView
        assert issubclass(resource, ResourceView), \
            "{0} not subclass of ResourceView".format(resource)

        # Cannot be abstract
        assert not resource.abstract, \
            "Attempt register of abstract resource: {0}.".format(resource)

        # Fabric of resources
        params = dict(self.params, **params)
        if params:
            params['name'] = resource.meta.name

            params['__module__'] = '%s.%s' % (
                self.prefix, self.str_version.replace('.', '_'))

            params['__doc__'] = resource.__doc__

            resource = type('%s%s' % (
                resource.__name__, len(self.resources)), (resource,), params)

        if self.resources.get(resource.meta.url_name):
            logger.warning(
                "A resource '%r' is replacing the existing record for '%s'",
                resource, self.resources.get(resource.meta.url_name))

        self.resources[resource.meta.url_name] = resource

    @property
    def urls(self):
        """ Provides URLconf details for the ``Api`` and all registered
            ``Resources`` beneath it.
        """
        urls = []

        for url_name in sorted(self.resources.keys()):

            resource = self.resources[url_name]
            urls.append(resource.as_url(
                api=self,
                name_prefix='-'.join(
                    (self.prefix, self.str_version)).strip('-'),
                url_prefix=self.str_version
            ))

        return patterns(self.prefix, *urls)

    def call(self, name, request=None, **params):
        """ Call resource by ``Api`` name.
        """
        if not name in self.resources:
            raise exceptions.HttpError('Unknown method \'%s\'' % name,
                                       status=status.HTTP_501_NOT_IMPLEMENTED)
        request = request or HttpRequest()
        resource = self.resources[name]
        view = resource.as_view(api=self)
        return view(request, **params)



from django.test import TestCase
from adrest.views import ResourceView


class CoreApiTest(TestCase):

    def test_base(self):
        from adrest.api import Api

        api = Api('1.0.0')
        self.assertEqual(api.version, '1.0.0')
        self.assertTrue(api.urls)

        class Resource(ResourceView):
            model = 'core.pirate'

        api.register(Resource)
        self.assertEqual(len(api.urls), 2)
        self.assertTrue(api.resources.get('pirate'))

        class TreasureResource(ResourceView):
            model = 'core.treasure'
            parent = Resource

        api.register(TreasureResource)
        self.assertEqual(len(api.urls), 3)
        self.assertTrue(api.resources.get('pirate-treasure'))

        class PrefixResource(TreasureResource):
            prefix = 'more'

        api.register(PrefixResource)
        self.assertEqual(len(api.urls), 4)
        self.assertTrue(api.resources.get('pirate-more-treasure'))



from .models import Task
from adrest.api import Api
from adrest.utils import emitter
from adrest.views import ResourceView


class TaskResource(ResourceView):
    emitters = emitter.XMLTemplateEmitter
    model = Task


class Task2Resource(ResourceView):
    allowed_methods = 'GET', 'POST'
    emit_include = 'description'
    emitters = emitter.JSONEmitter
    model = Task

    @staticmethod
    def to_simple__description(task, serializer):
        return "{0} -- {1}".format(task.title, task.user.username)

    def to_simple(self, content, simple, serializer=None):
        simple['api'] = self.api.version
        return simple


API = Api(version='1.0b')
API.register(TaskResource)
API.register(Task2Resource, url_name='task2', url_regex='task2')



from django.contrib.auth.models import User
from django.db import models


class Task(models.Model):
    title = models.CharField(max_length=100)
    user = models.ForeignKey(User)

    @property
    def username(self):
        return self.user.username



#!/usr/bin/env python
import os

from setuptools import setup, find_packages

from adrest import version, PROJECT, LICENSE


PACKAGE_DATA = []

for folder in ['templates']:
    for root, dirs, files in os.walk(os.path.join(PROJECT, folder)):
        for filename in files:
            PACKAGE_DATA.append("%s/%s" % (root[len(PROJECT) + 1:], filename))


def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__), fname)).read()
    except IOError:
        return ''

install_requires = read('requirements.txt').split()

setup(
    author_email='horneds@gmail.com',
    author='Kirill Klenov',
    description=read('DESCRIPTION'),
    install_requires=install_requires,
    license=LICENSE,
    long_description=read('README.rst'),
    name=PROJECT,
    package_data={'': PACKAGE_DATA},
    packages=find_packages(),
    platforms=('Any'),
    tests_require=['pymongo', 'milkman'],
    test_suite='tests.run_tests',
    url=' http://github.com/klen/{0}'.format(PROJECT),
    version=version,
)



from django.db import models


class Pirate(models.Model):
    name = models.CharField(max_length=50)


class Island(models.Model):
    title = models.CharField(max_length=50)


class Treasure(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    pirate = models.ForeignKey(Pirate, null=True, blank=True)
    island = models.ForeignKey(Island)



from django.test import TestCase

from adrest.views import ResourceView


class CoreResourceTest(TestCase):

    def test_meta(self):

        class AlphaResource(ResourceView):
            pass
        self.assertEqual(AlphaResource.meta.name, 'alpha')

        class BetaResource(ResourceView):
            name = 'beta'
        self.assertEqual(BetaResource.meta.name, 'beta')
        self.assertEqual(BetaResource.meta.url_name, 'beta')

        class GammaResource(ResourceView):
            name = 'gamma'
            parent = BetaResource

        self.assertEqual(GammaResource.meta.parents, [BetaResource])
        self.assertEqual(GammaResource.meta.name, 'gamma')
        self.assertEqual(GammaResource.meta.url_name, 'beta-gamma')



from milkman.dairy import milkman # nolint

from .api import API
from .models import Task
from adrest.tests.utils import AdrestTestCase


class SimpleTestCase(AdrestTestCase):
    api = API

    def test_base(self):
        self.assertEqual(self.api.str_version, '1.0b')

        uri = self.reverse('task')
        self.assertEqual(uri, '/simple/1.0b/task/')

        response = self.get_resource('task')
        self.assertContains(response, 'true')

        response = self.post_resource('task2', data=dict(title='new'))
        self.assertEqual(
            response.content,
            '{"api": "1.0b", "user": ["This field is required."]}')

        task = milkman.deliver(Task)
        response = self.get_resource('task2')
        self.assertContains(response, 'num_pages')
        self.assertContains(
            response,
            "{0} -- {1}".format(task.title, task.user.username))



from django.db import models
from django.test import TestCase
from django.views.generic import View

from adrest.mixin import HandlerMixin


class CoreHandlerTest(TestCase):

    def test_meta_model(self):

        class Resource(View, HandlerMixin):
            model = 'core.pirate'

        self.assertTrue(issubclass(Resource.model, models.Model))

    def test_meta_name(self):

        class Resource(View, HandlerMixin):
            model = 'core.pirate'

        self.assertEqual(Resource.meta.name, 'pirate')

        class IslandResource(View, HandlerMixin):
            model = 'core.island'
            name = 'map'

        self.assertEqual(IslandResource.meta.name, 'map')

        class TreasureResource(View, HandlerMixin):
            parent = Resource
            model = 'core.treasure'

        self.assertEqual(TreasureResource.meta.name, 'treasure')



import mimeparse
from django.http import HttpResponse

from ..utils import MetaOptions
from ..utils.emitter import JSONEmitter, BaseEmitter
from ..utils.paginator import Paginator
from ..utils.tools import as_tuple


__all__ = 'EmitterMixin',


class EmitterMeta(type):
    """ Prepare resource's emiters.
    """
    def __new__(mcs, name, bases, params):
        params['meta'] = params.get('meta', MetaOptions())
        cls = super(EmitterMeta, mcs).__new__(mcs, name, bases, params)

        cls.emitters = as_tuple(cls.emitters)
        assert cls.emitters, "Resource should have emitters."
        cls.meta.emitters_dict = dict(
            (e.media_type, e) for e in cls.emitters
        )

        for e in cls.emitters:
            assert issubclass(e, BaseEmitter), \
                "Emitter should be subclass of `adrest.utils.emitter.BaseEmitter`" # nolint

        return cls


class EmitterMixin(object):
    """ Serialize response.

        :param emitters: Emitter's choosen by http header
        :param emit_template: Force template name for template based emitters
        :param emit_fields: Manualy defined set of model fields for serializers
        :param emit_include: Additionaly fields for model serialization
        :param emit_exclude: Exclude fields from model serialization
        :param emit_related: Dict with field serialization options

        Example: ::

            class SomeResource():
                emit_fields = ['pk', 'user', 'customfield']
                emit_related = {
                    'user': {
                        fields: ['username']
                    }
                }

                def to_simple__customfield(self, user):
                    return "I'm hero! " + user.username

    """
    __metaclass__ = EmitterMeta

    emitters = JSONEmitter
    emit_exclude = None
    emit_fields = None
    emit_include = None
    emit_options = None
    emit_related = None
    emit_template = None

    def emit(self, content, request=None, emitter=None):
        """ Serialize response.
        """
        # Get emitter for request
        emitter = emitter or self.determine_emitter(request)
        emitter = emitter(self, request=request, response=content)

        # Serialize the response content
        response = emitter.emit()
        assert isinstance(response, HttpResponse), \
            "Emitter must return HttpResponse"

        # Append pagination headers
        if isinstance(content, Paginator):
            linked_resources = []
            if content.next_page:
                linked_resources.append('<{0}>; rel="next"'.format(
                    content.next_page))
            if content.previous_page:
                linked_resources.append(
                    '<{0}>; rel="previous"'.format(content.previous_page))
            response["Link"] = ", ".join(linked_resources)

        return response

    @staticmethod
    def to_simple(content, simple, serializer=None):
        """ Modify simple structure before response.
        """
        return simple

    @classmethod
    def determine_emitter(cls, request):
        " Return must fine emitter for request "
        default_emitter = cls.emitters[0]
        if not request:
            return default_emitter

        if request.method == 'OPTIONS':
            return JSONEmitter

        accept = request.META.get('HTTP_ACCEPT', '*/*')
        if accept == '*/*':
            return default_emitter

        base_format = mimeparse.best_match(cls.meta.emitters_dict.keys(),
                                           accept)
        return cls.meta.emitters_dict.get(
            base_format,
            default_emitter)



from adrest.views import ResourceView


class AuthorResource(ResourceView):
    model = 'main.author'


class BookResource(ResourceView):
    parent = AuthorResource
    model = 'main.book'
    allowed_methods = 'get', 'post', 'put'
